/****************************************************************************
** $Id: qc_applicationwindow.h,v 1.18 2003/10/03 18:26:49 pdamsten Exp $
**
** Copyright (C) 2001-2003 RibbonSoft. All rights reserved.
**
** This file is part of the QCad project.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding valid QCad Professional Edition licenses may use 
** this file in accordance with the qcadlib Commercial License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.ribbonsoft.com for further details.
**
** Contact info@ribbonsoft.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#ifndef QC_APPLICATIONWINDOW_H
#define QC_APPLICATIONWINDOW_H

#include <qaction.h>
#include <qlayout.h>
#include <qmainwindow.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qsplitter.h>
#include <qstatusbar.h>
#include <qtable.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qwhatsthis.h>
#include <qworkspace.h>

#include "rs_document.h"

#include "qc_dialogfactory.h"
#include "qc_graphicview.h"
#include "qc_mdiwindow.h"

#include "qg_actionhandler.h"
#include "qg_actionfactory.h"
#include "qg_blockwidget.h"
#include "qg_cadtoolbar.h"
#include "qg_commandwidget.h"
#include "qg_librarywidget.h"
#include "qg_layerwidget.h"
#include "qg_mainwindowinterface.h"
#include "qg_recentfiles.h"
#include "qg_pentoolbar.h"

//const QString QC_VERSION = "1.9.1";
//const QString QC_APPNAME = "QCad";
//const bool QC_DEBUGGING = true;

/**
 * Main application window. Hold together document, view and controls.
 *
 * @author Andrew Mustun
 */
class QC_ApplicationWindow: public QMainWindow,
    public QG_MainWindowInterface {
    Q_OBJECT

public:
    QC_ApplicationWindow();
    ~QC_ApplicationWindow();

    /** initializes all QActions of the application */
    void initActions();
    /** initMenuBar creates the menu_bar and inserts the menuitems */
    void initMenuBar();
    /**
     * this creates the toolbars. Change the toobar look and add new
     * toolbars in thisfunction 
     */
    void initToolBar();
    /** setup the statusbar */
    void initStatusBar();

    /** reads settings from rc file / registry */
    void initSettings();

    /** stores settings to rc file / registry */
    void storeSettings();

    /** Updates the file menu (recent files entries). */
    void updateRecentFilesMenu();

    /** setup the documents */
    void initMDI();
    /** setup the mainview*/
    void initView();

    /** overloaded for Message box on last window exit */
    bool queryExit(bool force);

	/** Catch hotkey for giving focus to command line. */
    virtual void keyPressEvent(QKeyEvent* e);

public slots:
	void slotFocus();
    void slotBack();
    //void slotNext();
    void slotEnter();
    void slotFocusCommandLine();
    void slotScriptsMenuAboutToShow();
    void slotScriptsMenuActivated(int);

    void slotWindowActivated(QWidget* w);
    void slotWindowsMenuAboutToShow();
    void slotWindowsMenuActivated(int);
    void slotTileHorizontal();

    void slotPenChanged(RS_Pen p);

    /** generates a new document for a graphic. */
    QC_MDIWindow* slotFileNew(RS_Document* doc=NULL);
    /** opens a document */
    void slotFileOpen();
    /**
    * opens a recent file document
    * @param id File Menu id of the file
    */
    void slotFileOpenRecent(int id);
    /**
     * opens the given file.
     */
    void slotFileOpen(const QString& fileName, RS2::FormatType type);
    /** saves a document */
    void slotFileSave();
    /** saves a document under a different filename*/
    void slotFileSaveAs();
	/** exports the document as bitmap */
	void slotFileExport();
	void slotFileExport(const QString& name, const QString& format, 
		QSize size, bool black, bool bw=false);
    /** closes the current file */
    void slotFileClose();
    /** closing the current file */
    void slotFileClosing();
    /** prints the current file */
    void slotFilePrint();
    /** shows print preview of the current file */
    void slotFilePrintPreview();
    /** exits the application */
    void slotFileQuit();

	/** forces to quit QCad (demo) */
	void slotFileDemoQuit();

    /** toggle the grid */
    void slotViewGrid(bool toggle);
    /** toggle the statusbar */
    //void slotViewStatusBar(bool toggle);
    /** toggle the layerlist */
    //void slotViewLayerList(bool toggle);
    /** toggle the blocklist */
    //void slotViewBlockList(bool toggle);
    /** toggle the command line */
    //void slotViewCommandLine(bool toggle);
    /** toggle the option toolbar */
    //void slotViewOptionToolbar(bool toggle);

    //void slotBlocksEdit();
    void slotOptionsGeneral();

    /** shows an about dlg*/
    void slotHelpAbout();

    /** dumps entities to file */
    void slotTestDumpEntities(RS_EntityContainer* d=NULL);
    /** updates all inserts */
    void slotTestUpdateInserts();
    /** draws some random lines */
    void slotTestDrawFreehand();
    /** inserts a test block */
    void slotTestInsertBlock();
    /** inserts a test ellipse */
    void slotTestInsertEllipse();
    /** inserts a test text */
    void slotTestInsertText();
    /** inserts a test image */
    void slotTestInsertImage();
    /** unicode table */
    void slotTestUnicode();
    /** math experimental */
    void slotTestMath01();
    /** python experimental */
    void slotTestPython();
    /** resizes window to 640x480 for screen shots */
    void slotTestResize640();
    /** resizes window to 640x480 for screen shots */
    void slotTestResize800();
    /** resizes window to 640x480 for screen shots */
    void slotTestResize1024();

signals:
    void windowsChanged(bool windowsLeft);

public:
    /**
     * @return Pointer to application window.
     */
    static QC_ApplicationWindow* getAppWindow() {
        return appWindow;
    }

    /**
     * @return Pointer to workspace.
     */
    QWorkspace* getWorkspace() {
        return workspace;
    }

    /**
     * @return Pointer to the currently active MDI Window or NULL if no
     * MDI Window is active.
     */
    QC_MDIWindow* getMDIWindow() {
        return (QC_MDIWindow*)workspace->activeWindow();
    }

    /**
     * Implementation from QG_ActionHandler.
     *
     * @return Pointer to the graphic view of the currently active document
     * window or NULL if no window is available.
     */
    virtual QG_GraphicView* getGraphicView() {
        QC_MDIWindow* m = getMDIWindow();
        if (m!=NULL) {
            return m->getGraphicView();
        }
        return NULL;
    }

    /**
     * Implementation from QG_ActionHandler.
     *
     * @return Pointer to the graphic document of the currently active document
     * window or NULL if no window is available.
     */
    virtual RS_Document* getDocument() {
        QC_MDIWindow* m = getMDIWindow();
        if (m!=NULL) {
            return m->getDocument();
        }
        return NULL;
    }

    /**
     * Implementation from QG_ActionHandler.
     *
     * @return Pointer to this.
     */
    virtual QMainWindow* getMainWindow() {
        return this;
    }

    /**
     * Implementation from QG_ActionHandler.
     *
     * @return Pointer to action handler.
     */
    virtual QG_ActionHandler* getActionHandler() {
        return actionHandler;
    }

	virtual void setFocus2() {
		setFocus();
	}

protected:
    void closeEvent(QCloseEvent*);
    virtual void mouseReleaseEvent(QMouseEvent* e);

private:
    /** Pointer to the application window (this). */
    static QC_ApplicationWindow* appWindow;

    /** Workspace for MDI */
    QWorkspace* workspace;

	/** Dialog factory */
	QC_DialogFactory* dialogFactory;

    /** Layer list widget */
    QG_LayerWidget* layerWidget;
    /** Block list widget */
    QG_BlockWidget* blockWidget;
    /** Library browser widget */
    QG_LibraryWidget* libraryWidget;
	
    /** Layer list dock widget */
    QDockWindow* layerDockWindow;
    /** Block list dock widget */
    QDockWindow* blockDockWindow;
    /** Library list dock widget */
    QDockWindow* libraryDockWindow;

	/** Command line */
	QG_CommandWidget* commandWidget;
	QDockWindow* commandDockWindow;

    /** Coordinate widget */
    QG_CoordinateWidget* coordinateWidget;
    /** Mouse widget */
    QG_MouseWidget* mouseWidget;
    /** Selection Status */
    QG_SelectionWidget* selectionWidget;

    /** Option widget for individual tool options */
    QToolBar* optionWidget;

    /** Recent files list */
    QG_RecentFiles* recentFiles;

    /** Action handler. */
    QG_ActionHandler* actionHandler;

    QPopupMenu* fileMenu;
    QPopupMenu* windowsMenu;
    QPopupMenu* scriptsMenu;
    QPopupMenu* helpMenu;
    QPopupMenu* testMenu;

    /** the main toolbars */
    QToolBar* fileToolBar;
    QToolBar* editToolBar;
    QToolBar* zoomToolBar;

    // Toolbar for selecting the current pen
    QG_PenToolBar* penToolBar;
    // Toolbar for CAD tools
    QG_CadToolBar* cadToolBar;

    QAction* helpAboutApp;

    QAction *testDumpEntities;
    QAction *testUpdateInserts;
    QAction *testDrawFreehand;
    QAction *testInsertBlock;
    QAction *testInsertText;
    QAction *testInsertImage;
    QAction *testUnicode;
    QAction *testInsertEllipse;

    QAction *testMath01;

    QAction *testPython;

    QAction *testResize640;
    QAction *testResize800;
    QAction *testResize1024;

};


#endif

