/****************************************************************************
** $Id: rs_export.cpp,v 1.4 2003/09/15 12:58:36 andrew Exp $
**
** Copyright (C) 2001-2003 RibbonSoft. All rights reserved.
**
** This file is part of the qcadlib Library project.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding valid qcadlib Professional Edition licenses may use 
** this file in accordance with the qcadlib Commercial License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.ribbonsoft.com for further details.
**
** Contact info@ribbonsoft.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/


#include "rs_export.h"

#include "rs_filterdxf.h"
#include "rs_filtercxf.h"
#include "rs_graphic.h"


/**
 * Default constructor.
 *
 * @param graphic The container which contains the 
 *        entities we're trying to export.
 *        Usually that's an RS_Graphic entity but
 *        it can also be a polyline, text, group, ...
 */
RS_Export::RS_Export(RS_Graphic& graphic) {
	RS_DEBUG->print("RS_Export::RS_Export");
    this->graphic = &graphic;
}


/**
 * Calls the import method of the object responsible for the format
 * of the given file.
 *
 * @param file Path and name of the file to import.
 */
void RS_Export::fileExport(const RS_String& file) {

	if (graphic==NULL) {
		std::cerr << "RS_Export::fileExport: graphic pointer NULL\n";
		return;
	}

	//std::cout << "RS_Export::fileExport\n";

    RS_DEBUG->print("RS_Export::fileExport");
    //RS_DEBUG->print("Trying to export file '%s'...", file.latin1());

    RS_String extension;
    extension = RS_FileInfo(file).extension(false).lower();

	//std::cout << "  extension: " << extension << "\n";

    // Export DXF file:
    if (extension=="dxf") {
		// This is weird but RS_FilterDXF filter() crashes under 
		//  Winboze XP, so let's allocate with new:
		//
        RS_FilterDXF* filter = new RS_FilterDXF(graphic);
        if (filter->canExport("dxf")) {
            filter->fileExport(file);
        }
		delete filter;
    } else if (extension=="cxf") {
        RS_FilterCXF* filter = new RS_FilterCXF(graphic);
        if (filter->canExport("cxf")) {
            filter->fileExport(file);
        }
		delete filter;
    }
}


// EOF
